import os

from os import path
from datetime import datetime


def scan_folder(src, extensions, type_):
    files = []
    exts = []

    # Ensure all extensions are in lower case
    for ext in extensions:
        exts.append(ext.lower())

    for f in os.listdir(src):
        fullpath = path.join(src, f)
        extension = path.splitext(f)[-1].lower()

        if path.isdir(fullpath):
            files.extend(scan_folder())

        if extension in exts:
            created_at = datetime.fromtimestamp(
                os.stat(fullpath).st_ctime
            ).strftime("%Y-%m-%d")

            files.append((fullpath, created_at, type_))

    return files



from . import Camera


class SonyNex(Camera):
    img_folders = ["DCIM/*MSDCF"]
    raw_folders = img_folders
    vid_folders = ["PRIVATE/M4ROOT/CLIP"]

    # img_extensions = [".JPG"]
    img_extensions = []
    # raw_extensions = [".ARW"]
    raw_extensions = []
    vid_extensions = [".MP4", ".XML"]



class Camera:
    """Abstract class of Camera."""

    img_folders = None
    raw_folders = None
    vid_folders = None

    img_extensions = None
    raw_extensions = None
    vid_extensions = None


def get_all_cameras():
    import pkgutil, importlib, inspect

    cameras = {}

    for _, modname, _ in pkgutil.walk_packages(
        path=__path__, prefix=__name__ + ".", onerror=lambda x: None
    ):
        modulesource = importlib.import_module(modname)
        for module in inspect.getmembers(modulesource, inspect.isclass):
            class_name = module[0]

            if not cameras.get(class_name):
                cameras[class_name] = module[1]

    # Pop the 'Camera' abstract class
    cameras.pop('Camera', None)

    return cameras




import argparse
import signal

from cambak.backup import backup_camera
from cambak.cameras import get_all_cameras


def keyboardInterruptHandler(signal, frame):
    print("KeyboardInterrupt has been caught. Cleaning up...")
    exit(125)


def main_cli():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "src", help="Source folder (mounted card/usb camera volume)", type=str
    )

    parser.add_argument(
        "dest", help="Destination folder (local, network volume)", type=str
    )

    parser.add_argument(
        "-t", "--type", help="Type of camera", type=str, required=True
    )
    parser.add_argument(
        "-n", "--name", help="Name of the camera", type=str, required=True
    )

    parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        help="Override if file already exists in the dest folder",
    )

    args = parser.parse_args()
    cameras_types = get_all_cameras()

    signal.signal(signal.SIGINT, keyboardInterruptHandler)
    backup_camera(
        args.src,
        args.dest,
        args.name,
        cameras_types.get(args.type, None),
        args.force,
    )


if __name__ == "__main__":
    main_cli()



import copy as cp

from os import path, mkdir
from glob import glob
from shutil import copy

from .cameras import Camera
from .scanner import scan_folder


def backup_camera(
    src: str,
    dest: str,
    name: str,
    type_: Camera,
    force: bool = False,
    **kwargs
):
    def get_fullpaths(paths):
        existing_files = []

        for p in paths:
            fullpaths = glob(path.join(src, p))

            for fullpath in fullpaths:
                if path.isdir(fullpath):
                    existing_files.append(fullpath)
                else:
                    print("%s not exists. Skipping it.")

        return existing_files

    files = []
    dates = set()
    src_path = {}

    # Generate folder pictures fullpaths
    src_path["Pictures"] = get_fullpaths(type_.img_folders)
    src_path["RAW"] = get_fullpaths(type_.raw_folders)
    src_path["Videos"] = get_fullpaths(type_.vid_folders)

    # Check if the dest folder exists
    if path.isdir(dest) is False:
        raise IOError("Destination folder doesn't exists.")

    # Get all files categorized
    for folder_type, folders in src_path.items():
        if folder_type == "Pictures":
            extensions = type_.img_extensions
        elif folder_type == "RAW":
            extensions = type_.raw_extensions
        elif folder_type == "Videos":
            extensions = type_.vid_extensions

        for p in folders:
            files.extend(scan_folder(p, extensions, folder_type))

    # Get all dates and create all folders
    for _, date, _ in files:
        if date not in dates:
            dates.add(date)
            date_path = path.join(dest, date)

            if not path.exists(date_path):
                mkdir(date_path)
                mkdir(path.join(date_path, name))
                mkdir(path.join(date_path, name, "Pictures"))
                mkdir(path.join(date_path, name, "RAW"))
                mkdir(path.join(date_path, name, "Videos"))

    total_files = len(files)
    remaining_files = cp.copy(total_files)
    print("Total of %s files to copy." % total_files)

    # Copy all files in their correct folders
    for f, date, f_type in files:
        remaining_files -= 1
        dest_path = path.join(dest, date, name, f_type)
        basename = path.basename(path.join(dest_path, f))

        if (force is False and not path.exists(basename)) or (force is True):
            try:
                print(
                    "Copying %s - Remaining: %s/%s"
                    % (f, remaining_files, total_files)
                )
                copy(f, dest_path)
            except Exception as err:
                print(
                    "!!! Unable to copy %r file. Cause: \n\t%s" % (f, str(err))
                )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cambak', 'cambak.cameras']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml>=5.3,<6.0']

setup_kwargs = {
    'name': 'cambak',
    'version': '0.1.0',
    'description': 'Camera Backup script',
    'long_description': None,
    'author': 'Michael Vieira',
    'author_email': 'contact@mvieira.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
